import Flutter
import Foundation
import AMRSDK

public class AMRInterstitialPlugin: NSObject, FlutterPlugin {

    fileprivate var interstitialList: [Int: AMRInterstitial] = [:]
    fileprivate var delegateList: [Int: AMRInterstitialDelegate] = [:]
    fileprivate var pluginRegistrar: FlutterPluginRegistrar?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = AMRInterstitialPlugin()
        instance.pluginRegistrar = registrar
        let channel = FlutterMethodChannel(name: "admost_flutter/interstitial", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String : Any]  else {
            result(FlutterError(code: "Missing args!", message: "Unable to convert args to [String : Any]", details: nil))
            return
        }
        let id = args["id"] as? Int ?? 0
        let zoneId = args["zoneId"] as? String ?? ""
        
        switch call.method {
        case "setListener":
            if let interstitial = interstitialList[id] {
                let channel = FlutterMethodChannel(name: "admost_flutter/interstitial_\(id)", binaryMessenger: pluginRegistrar!.messenger())
                delegateList[id] = AMRIntersitialPluginDelegate(channel: channel)
                interstitial.delegate = delegateList[id]
            }
            break
        case "load":
            let interstitial = AMRInterstitial(forZoneId: zoneId)
            interstitialList[id] = interstitial
            interstitial!.load()
            result(nil)
            break
        case "isLoaded":
            if let interstitial = interstitialList[id] {
                result(interstitial.isLoaded)
            }
            result(false)
            break
        case "show":
            if let interstitial = interstitialList[id] {
                if interstitial.isLoaded, let rootViewController = UIApplication.shared.keyWindow?.rootViewController {
                    interstitial.show(from: rootViewController)
                }
            }
            break
        case "dispose":
            interstitialList.removeValue(forKey: id)
            delegateList.removeValue(forKey: id)
            break
        default:
            result(FlutterMethodNotImplemented)
        }
    }
}

class AMRIntersitialPluginDelegate: NSObject, AMRInterstitialDelegate {
    let channel: FlutterMethodChannel
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    func didReceive(_ interstitial: AMRInterstitial!) {
        channel.invokeMethod("loaded", arguments: nil)
    }
    
    func didFail(toReceive interstitial: AMRInterstitial!, error: AMRError!) {
        channel.invokeMethod("failedToLoad", arguments:  [
            "errorCode": error.errorCode,
            "error": error.errorDescription
        ])
    }
    
    func didShow(_ interstitial: AMRInterstitial!) {
        channel.invokeMethod("opened", arguments: nil)
    }
    
    func didFail(toShow interstitial: AMRInterstitial!, error: AMRError!) {
        channel.invokeMethod("failedToLoad", arguments:  [
            "errorCode": error.errorCode,
            "error": error.errorDescription
        ])
    }
    
    func didClick(_ interstitial: AMRInterstitial!) {
        channel.invokeMethod("clicked", arguments: nil)
    }
    
    func didDismiss(_ interstitial: AMRInterstitial!) {
        channel.invokeMethod("dismissed", arguments: nil)
    }
}
