import Flutter
import Foundation
import AMRSDK

public class AMRRewardedVideoPlugin: NSObject, FlutterPlugin {

    fileprivate var rewardedVideoList: [Int: AMRRewardedVideo] = [:]
    fileprivate var delegateList: [Int: AMRRewardedVideoDelegate] = [:]
    fileprivate var pluginRegistrar: FlutterPluginRegistrar?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = AMRRewardedVideoPlugin()
        instance.pluginRegistrar = registrar
        let channel = FlutterMethodChannel(name: "admost_flutter/rewarded", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let args = call.arguments as? [String : Any]  else {
            result(FlutterError(code: "Missing args!", message: "Unable to convert args to [String : Any]", details: nil))
            return
        }
        let id = args["id"] as? Int ?? 0
        let zoneId = args["zoneId"] as? String ?? ""
        
        switch call.method {
        case "setListener":
            if let interstitial = rewardedVideoList[id] {
                let channel = FlutterMethodChannel(name: "admost_flutter/rewarded_\(id)", binaryMessenger: pluginRegistrar!.messenger())
                delegateList[id] = AMRRewardedVideoPluginDelegate(channel: channel)
                interstitial.delegate = delegateList[id]
            }
            break
        case "load":
            let rewardedVideo = AMRRewardedVideo(forZoneId: zoneId)
            rewardedVideoList[id] = rewardedVideo
            rewardedVideo!.load()
            result(nil)
            break
        case "isLoaded":
            if let rewardedVideo = rewardedVideoList[id] {
                result(rewardedVideo.isLoaded)
            }
            result(false)
            break
        case "show":
            if let rewardedVideo = rewardedVideoList[id] {
                if rewardedVideo.isLoaded, let rootViewController = UIApplication.shared.keyWindow?.rootViewController {
                    rewardedVideo.show(from: rootViewController)
                }
            }
            break
        case "dispose":
            rewardedVideoList.removeValue(forKey: id)
            delegateList.removeValue(forKey: id)
            break
        default:
            result(FlutterMethodNotImplemented)
        }
    }
}

class AMRRewardedVideoPluginDelegate: NSObject, AMRRewardedVideoDelegate {
    let channel: FlutterMethodChannel
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    func didReceive(_ rewardedVideo: AMRRewardedVideo!) {
        channel.invokeMethod("loaded", arguments: nil)
    }
    
    func didFail(toReceive rewardedVideo: AMRRewardedVideo!, error: AMRError!) {
        channel.invokeMethod("failedToLoad", arguments:  [
            "errorCode": error.errorCode,
            "error": error.errorDescription
        ])
    }
    
    func didShow(_ rewardedVideo: AMRRewardedVideo!) {
        channel.invokeMethod("opened", arguments: nil)
    }
    
    func didFail(toShow rewardedVideo: AMRRewardedVideo!, error: AMRError!) {
        channel.invokeMethod("failedToLoad", arguments:  [
            "errorCode": error.errorCode,
            "error": error.errorDescription
        ])
    }
    
    func didComplete(_ rewardedVideo: AMRRewardedVideo!) {
        channel.invokeMethod("completed", arguments: nil)
    }
    
    func didClick(_ rewardedVideo: AMRRewardedVideo!) {
        channel.invokeMethod("clicked", arguments: nil)
    }
    
    func didDismiss(_ rewardedVideo: AMRRewardedVideo!) {
        channel.invokeMethod("dismissed", arguments: nil)
    }
}
