import Foundation
import AMRSDK

class AdmostBanner : NSObject, FlutterPlatformView {

    private let channel: FlutterMethodChannel
    private let messeneger: FlutterBinaryMessenger
    private let frame: CGRect
    private let viewId: Int64
    private let args: [String: Any]
    private var adView: AMRBanner?
    private var bannerContainer: UIView?

    init(frame: CGRect, viewId: Int64, args: [String: Any], messeneger: FlutterBinaryMessenger) {
        self.args = args
        self.messeneger = messeneger
        self.frame = frame
        self.viewId = viewId
        channel = FlutterMethodChannel(name: "admost_flutter/banner_\(viewId)", binaryMessenger: messeneger)
    }
    
    func view() -> UIView {
        return getBannerAdView()
    }

    fileprivate func dispose() {
        adView?.bannerView.removeFromSuperview()
        adView = nil
        channel.setMethodCallHandler(nil)
    }
    
    fileprivate func getBannerAdView() -> UIView! {
        if adView == nil {
            adView = AMRBanner(forZoneId: self.args["adUnitId"] as? String ?? "")
            let size = getSize()
            
            bannerContainer = UIView(frame: CGRect(x: 0, y: 0, width: size.width, height: size.height))
            channel.setMethodCallHandler { [weak self] (flutterMethodCall: FlutterMethodCall, flutterResult: FlutterResult) in
                switch flutterMethodCall.method {
                case "setListener":
                    self?.adView?.delegate = self
                    break
                case "dispose":
                    self?.dispose()
                    break
                default:
                    flutterResult(FlutterMethodNotImplemented)
                }
            }
            adView?.load()
        }
        
        return bannerContainer
    }
    
    fileprivate func getSize() -> CGSize {
        let size = args["adSize"] as? [String: Any]
        let width = size!["width"] as? Double ?? 0
        let height = size!["height"] as? Double ?? 0
        
        return CGSize(width: width, height: height)
    }
}

extension AdmostBanner : AMRBannerDelegate {
    func didReceive(_ banner: AMRBanner!) {
        bannerContainer?.addSubview(banner.bannerView)
        channel.invokeMethod("loaded", arguments: nil)
    }
    
    func didFail(toReceive banner: AMRBanner!, error: AMRError!) {
        channel.invokeMethod("failedToLoad", arguments:  [
            "errorCode": error.errorCode,
            "error": error.errorDescription
        ])
    }
    
    func didClick(_ banner: AMRBanner!) {
        channel.invokeMethod("clicked", arguments: nil)
    }
}
